    function [x, err] = bissection(f, x1, x2, nmax, epsilon, resul)
%
%  Méthode de la bissection en une variable pour résoudre f(x) = 0     
%  Programmeur: A. Fortin 
%  Référence: Analyse numérique pour ingénieurs, A. Fortin,
%             Presses internationales Polytechnique, 
%             Cinquième édition, 2015
%             Section 2.2
%
%  function [x, err] = bissection(f, x1, x2, nmax, epsilon, resul) 
%  Arguments
%  Entrée:
%  1) f: Le nom entre apostrophes (' ') du fichier .m contenant la fonction
%        f(x). Par exemple  'fonc' correspondrait à un fichier fonc.m. 
%        On peut aussi donner le nom (sans apostrophes) d'une fonction 
%        anonyme comme par exemple 
%        f_anonyme = @(x) x*x - 2.;
%  2) x1: La borne inférieure de l'intervalle initial.
%  3) x2: La borne supérieure de l'intervalle initial.
%  4) nmax: Le nombre maximal d'itérations.
%  5) epsilon: Le critère d'arrêt.
%  6) resul: (facultatif) Si vous voulez les résultats dans un fichier, 
%     vous devez fournir le nom entre apostrophes (' ') de ce fichier.
%     Exemple: 'resul.dat' qui créera un fichier nommé resul.dat.
%  Sortie:
%  1) x:  vecteur contenant les différentes itérations.
%  2) err: S'il y a convergence, err est le vecteur dont les composantes 
%     sont |x_n - r|, r étant la racine trouvée. 
%     Sinon, err est une matrice vide.
%
%  Exemples d'appel:
%  Si vous avez un fichier fonction (par exemple fonc.m)
%  [x, err] = bissection('fonc', 1, 2, 20, 1.0e-5)
%  [x, err] = bissection('fonc', 1, 2, 20, 1.0e-5, 'resul.dat')
%   
%  Si vous avec une fonction anonyme (@) (on ne met pas les apostrophes)
%  [x, err] = bissection( f_anonyme, 1, 2, 20, 1.0e-5)
%  [x, err] = bissection( f_anonyme, 1, 2, 20, 1.0e-5, 'resul.dat')
%  
    g = f;
    f=fcnchk(f);
    fx1 = f(x1);
    fx2 = f(x2);
    X1=zeros(1); X2=zeros(1); x=zeros(1); Fxm=zeros(1);
    X1(1) = x1;
    X2(1) = x2;
      if fx1*fx2 > 0
           fprintf('Possibilité qu''il n''y ait pas de changement de signe dans\n');
           fprintf('l''intervalle initial fourni\r\n\r\n');
           return; 
      end
    xm  = (x1+x2)/2;
    x(1) = xm;
    fxm = f(xm);
    Fxm(1) = fxm;
    for n=1:nmax
       if  fxm == 0
           x1 = xm;
           x2 = xm;
       elseif  fx2*fxm > 0
           x2 = xm;
           fx2 = fxm;
       else
           x1 = xm;
       end
       X1(n+1) = x1;
       X2(n+1) = x2;
       xm = (x1+x2)/2;
       x(n+1) = xm;
       fxm = f(xm);
       Fxm(n+1) = fxm;
       tol = abs(x2-x1);
       tolrel = tol/(2*abs(xm)+eps);
       if (tolrel < epsilon)
           break;
       end
    end
    if (tolrel < epsilon)
      err = abs(x - xm);
      converg = 1;
    else
      err = [];
      converg = 0;
      fprintf('\r\nIl n''y a pas de convergence après %1.0f itérations.\r\n\r\n', nmax);
    end
    if (nargin > 5)
      bissection_out(g, resul, X1, X2, x, Fxm, nmax, epsilon, converg);
    end
