    function I = int_boole(f, a, b, N)
%
%  Méthode de Boole composée
%  Programmeur: A. Lioret 
%  Référence: Analyse numérique pour ingénieurs, A. Fortin,
%             Presses internationales Polytechnique, 
%             Cinquième édition, 2015
%             Section 6.4.1
%
%  function I = int_boole(f, a, b, N)
%  Arguments
%
%  Entrée:
%  1) f: Le nom entre apostrophes (' ') du fichier .m 
%     contenant la fonction à intégrer, 
%     ou encore 
%     le nom d'une fonction anonyme comme par exemple
%     f_anonyme = @(x) x.*x -2;
%     ou encore
%     les valeurs de la fonction aux points d'intégration 
%     dans un vecteur ligne (f_vecteur), correspondant avec
%     le nombre de sous-intervalles utilisés (si on a N sous-intervalles,
%     on doit avoir N+1 valeurs de la fonction à intégrer).
%     (N est un multiple positif de 4).
%  2) a: La borne inférieure de l'intervalle d'intégration.
%  3) b: La borne supérieure de l'intervalle d'intégration.
%  4) N: Le nombre de sous-intervalles utilisés.
%     Note: N doit être un multiple positif de 4.
%  Sortie:
%  1) I: Valeur calculée de l'intégrale définie. 
%
%  Préalable:
%  Si vous connaissez la fonction à intégrer, vous pouvez créer
%  un fichier .m contenant cette fonction.
%  Voir par exemple le fichier fonc.m. Alternativement, vous pouvez aussi
%  utiliser une fonction anonyme ou un vecteur.
%
%  Exemples d'appel:
%  I = int_boole('fonc',    0, 2, 8)
%  I = int_boole(f_anonyme, 0, 2, 8)
%  I = int_boole(f_vecteur, 0, 2, 8)
%
    if (rem(N,4) ~= 0)||(N <= 0)
      error('N doit être un multiple positif de 4');
    end
    h = (b - a)/N;

    if (ischar(f))
      x = a:h:b;
      fx = feval(f, x);
      if ~(all(isfinite(fx)))
        error('La fonction n''est pas définie pour certains points évalués');
      end
    else
      l = length(f);
      if (l ~= N+1)
        error('Le nombre de points fournis ne correspond pas avec le nombre de sous-intervalles');
      end
      fx = f;
    end
    v1 = 2:4:N;
    v2 = 3:4:N;
    v3 = 4:4:N;
    v4 = 5:4:N;
    yi0 = 7*(fx(1) + fx(N+1));
    yi1 = fx(v1);
    yi2 = fx(v2);
    yi3 = fx(v3);
    yi4 = fx(v4);

    I = (2*h/45)*(yi0 + 32*sum(yi1) + 12*sum(yi2) + 32*sum(yi3) + 14*sum(yi4));
