    function [I, ti, wi] = int_gauss(f, a, b, n, resul)
%
%  Quadrature de Gauss
%  Programmeur: A. Lioret 
%  Référence: Analyse numérique pour ingénieurs, A. Fortin,
%             Presses internationales Polytechnique, 
%             Cinquième édition, 2015
%             Section 6.4.3
%
%  function [I, ti, wi] = int_gauss(f, a, b, n, resul)
%
%  Arguments
%  Entrée:
%  1) f: Le nom entre apostrophes (' ') du fichier .m contenant la fonction 
%     f(x) comme par exemple  'fonc' (correspondant à un fichier fonc.m) 
%     ou encore le nom de la fonction anonyme comme par exemple 
%     f_anonyme = @(x) x*x - 2.;
%  2) a: La borne inférieure de l'intervalle d'intégration.
%  3) b: La borne supérieure de l'intervalle d'intégration.
%  4) n: Le nombre de points à utiliser dans la formule de quadrature.
%  5) resul: (facultatif) Si vous voulez les résultats dans un fichier, 
%     vous devez fournir le nom entre apostrophes (' ') de ce fichier.
%     Exemple: 'resul.dat' qui créera un fichier nommé resul.dat.
%  Retour:
%  1) I:  Valeur calculée de l'intégrale définie.
%  2) ti: Vecteur contenant les points d'intégration.
%  3) wi: Vecteur contenant les poids d'intégration. 
%
%  Préalable:
%  Vous devez créer un fichier .m contenant la fonction à intégrer.
%  Voir par exemple le fichier fonc.m. Alternativement, vous pouvez aussi
%  utiliser une fonction anonyme.
%
%  Exemples d'appel:  
%  [I, ti, wi] = int_gauss('fonc', 0, 2, 5)
%  [I, ti, wi] = int_gauss('fonc', 0, 2, 5, 'resul.dat')
%
%  Si vous avec une fonction anonyme (@) (on ne met pas les apostrophes)
%  [I, ti, wi] = int_gauss(f_anonyme, 0, 2, 5)
%  [I, ti, wi] = int_gauss(f_anonyme, 0, 2, 5, 'resul.dat')

    g=f;
    f=fcnchk(f);
    if (n == 1)
      ti = 0;
      wi = 2;
    else
      u = (1:n-1)./sqrt((2*(1:n-1)).^2-1);
      [vc, ti] = eig(diag(u,-1) + diag(u,1));
      [ti, k] = sort(diag(ti));
      wi = 2*vc(1,k)'.^2;
    end
    xi = (a+b)/2 + (b-a)/2*ti;
    fxi = f(xi)*(b-a)/2;
    if (any(~isfinite(fxi)))
      error('La fonction n''est pas définie en certains points évalués');
    end 
    I = wi(:)'*fxi(:);

    if (nargin > 4)
      int_gauss_out(g, resul, I, ti, wi, a, b, n);
    end
