    function T = int_romberg(f, a, b, N, resul)
%
%  Méthode de Romberg
%  Programmeur: A. Lioret 
%  Référence: Analyse numérique pour ingénieurs, A. Fortin,
%             Presses Polytechnique internationales, 
%             Cinquième édition, 2015
%             Section 6.4.2
%
%  function T = int_romberg(f, a, b, N, resul)
%  Arguments
%  Entrée:
%  1) f: Vous devez fournir soit le nom entre apostrophes (' ') 
%     du fichier .m  contenant la fonction à intégrer, 
%     soit une fonction anonyme comme par exemple
%     f_anonyme = @(x) x.*x -2;
%     soit les valeurs de la fonction aux points d'intégration 
%     dans un vecteur ligne (f_vecteur), correspondant avec
%     le nombre de sous-intervalles utilisés (si on a N sous-intervalles,
%     on doit avoir N+1 valeurs de la fonction à intégrer).
%     (N doit être une puissance de 2).
%  2) a: La borne inférieure de l'intervalle d'intégration.
%  3) b: La borne supérieure de l'intervalle d'intégration.
%  4) N: Le nombre de sous-intervalles utilisés.
%     Note: N doit être une puissance de 2.
%  5) (facultatif) Si vous voulez les résultats dans un fichier,  
%     vous devez fournir le nom entre apostrophes (' ') de ce fichier.
%     Exemple: 'resul.dat' qui créera un fichier nommé resul.dat.
%  Sortie:
%  1) T: Matrice contenant les T(i,j), les approximations 
%     par extrapolation de Richardson de l'intégrale définie.  
%
%  Préalable:
%  Si vous connaissez la fonction à intégrer, vous pouvez créer
%  un fichier .m contenant cette fonction.
%  Voir par exemple le fichier fonc.m. Alternativement, vous pouvez aussi
%  utiliser une fonction anonyme ou un vecteur.
%
%  Exemples d'appel:
%  I = int_romberg('fonc',    0, 2, 8)
%  I = int_romberg(f_anonyme, 0, 2, 8)
%  I = int_romberg(f_vecteur, 0, 2, 8)
%
    g=f;
    n = fix(log(N)/log(2));
    if (n ~= log(N)/log(2))
      error('Le nombre de sous-intervalles choisi n''est pas une puissance de 2');
    else
      n = n+1;
    end
    T = zeros(n);
    if (ischar(f) || isa(f, 'function_handle') == 1)
      f=fcnchk(f);
      for k=1:n
        Nk = 2^(k-1);
        T(1,k) = int_trapezes(f, a, b, Nk);
      end
    else
      l = length(f) - 1;
      if (l ~= N)
        error('Le nombre de points fournis ne correspond pas avec le nombre de sous-intervalles');
      end
      for k=1:n
        Nk = 2^(k-1);
        j = 2^(n-k);
        ff = f(1:j:l+1);
        T(1,k) = int_trapezes(ff, a, b, Nk);      
      end
    end 
    for k=2:n
      m = 2*(k-1);
      for j=1:n-k+1
        T(k,j) = (2^m*T(k-1,j+1) - T(k-1,j))/(2^m - 1);
      end
    end

    if (nargin > 4)
      int_romberg_out(g, resul, T, a, b, n, N);
    end
