    function I = int_simpson13(f, a, b, N)
%
%  Méthode de Simpson 1/3 composée
%  Programmeur: A. Lioret 
%  Référence: Analyse numérique pour ingénieurs, A. Fortin,
%             Presses internationales Polytechnique, 
%             Cinquième édition, 2015
%             Section 6.4.1
%
%  function I = int_simpson13(f, a, b, N)
%  Arguments
%
%  Entrée:
%  1) f: Vous devez fournir soit le nom entre apostrophes (' ') du 
%     fichier .m  contenant la fonction à intégrer, 
%     soit une fonction anonyme comme par exemple
%     f_anonyme = @(x) x.*x -2;
%     soit les valeurs de la fonction aux points d'intégration 
%     dans un vecteur ligne (f_vecteur), correspondant avec
%     le nombre de sous-intervalles utilisés (si on a N sous-intervalles,
%     on doit avoir N+1 valeurs de la fonction à intégrer).
%     (N est un multiple positif de 2).
%  2) a: La borne inférieure de l'intervalle d'intégration.
%  3) b: La borne supérieure de l'intervalle d'intégration.
%  4) N: Le nombre de sous-intervalles utilisés.
%     Note: N doit être un multiple positif de 2.
%  Sortie:
%  1) I: Valeur calculée de l'intégrale définie. 
%
%  Préalable:
%  Si vous connaissez la fonction à intégrer, vous pouvez créer
%  un fichier .m contenant cette fonction.
%  Voir par exemple le fichier fonc.m. Alternativement, vous pouvez aussi
%  utiliser une fonction anonyme ou un vecteur.
%
%  Exemples d'appel:
%  I = int_simpson13('fonc',    0, 2, 4)
%  I = int_simpson13(f_anonyme, 0, 2, 4)
%  I = int_simpson13(f_vecteur, 0, 2, 4)

    if (rem(N,2) ~= 0)||(N <= 0)
      error('N doit être un multiple positif de 2');
    end
    h = (b - a)/N;

    if (ischar(f) || isa(f, 'function_handle') == 1)
      f=fcnchk(f);
      x = a:h:b;
      fx = f(x);
      if ~(all(isfinite(fx)))
        error('La fonction n''est pas définie en certains points évalués');
      end
    else
      l = length(f);
      if (l ~= N+1)
        error('Le nombre de points fournis ne correspond pas avec le nombre de sous-intervalles');
      end
      fx = f;
    end
    v1 = 2:2:N;
    v2 = 3:2:N;
    yi0 = fx(1) + fx(N+1);
    yi1 = fx(v1);
    yi2 = fx(v2);

    I = h/3*(yi0 + 4*sum(yi1) + 2*sum(yi2));
