    function  [x, err] = newton(f, df, x0, nmax, epsilon, resul)
%
%  Méthode de Newton en une variable pour résoudre f(x) = 0
%  Programmeur: A. Fortin 
%  Référence: Analyse numérique pour ingénieurs, A. Fortin,
%             Presses internationales Polytechnique, 
%             Cinquième édition, 2015
%             Section 2.4
%
%  function  [x, err] = newton(f, df, x0, nmax, epsilon, resul)
%  Arguments
%
%  Entrée:
%  1) f: Le nom entre apostrophes (' ') du fichier .m contenant la fonction 
%     f(x) comme par exemple  'fonc' (correspondant à un fichier fonc.m) 
%     ou encore le nom de la fonction anonyme comme par exemple 
%     f_anonyme = @(x) x*x - 2.;
%  2) df: mêmes indications qu'en 1) mais pour la dérivée de la fonction;
%  3) x0: L'estimé initial de la racine.
%  4) nmax: Le nombre maximal d'itérations.
%  5) epsilon: Le critère d'arrêt.
%  6) resul: (facultatif) Si vous voulez les résultats dans un fichier, vous
%     devez fournir le nom entre apostrophes (' ') de ce fichier.
%     Exemple: 'resul.dat' qui créera un fichier nommé resul.dat.
%  Sortie:
%  1) x: Vecteur contenant les différentes itérations.
%  2) err: S'il y a convergence, err est le vecteur dont les composantes 
%     sont |x_n - r|, r étant la racine trouvée. 
%     Sinon, err est une matrice vide.
%
%  Exemples d'appel:
% 
%  [x, err] = newton('fonc', 'derivee_fonc', 2.0, 10, 1.0e-6)
%  [x, err] = newton('fonc', 'derivee_fonc', 2.0, 10, 1.0e-6, 'resul.dat')
%
%  Si vous avec une fonction anonyme (@) (on ne met pas les apostrophes)
%  [x, err] = newton(f_anonyme, df_anonyme, 2.0, 10, 1.0e-6)
%  [x, err] = newton(f_anonyme, df_anonyme, 2.0, 10, 1.0e-6, 'resul.dat')

    g = f;
    dg = df;
    f=fcnchk(f);
    df=fcnchk(df);
    infini = 0;
    fx = zeros(1); x = zeros(1);
    x(1) = x0;
    fx0 = f(x0);
    if (~isfinite(fx0))
      error('f(x0) n''est pas un nombre fini.');
    end
    fx(1) = fx0;
    for n=1:nmax
         ind = n-1;
         dfx0 = df(x0);
         if (dfx0 ~= 0)            
            x0 = x0 - fx0/(dfx0+eps);
         elseif (n == 1)
            error('f''(x0) = 0');
         end
         fx0 = f(x0);
         if (~isfinite(fx0))
           if (n == 1)
             fprintf('\r\n Il n''y a pas convergence.\r\n\r\n');
           else
             fprintf('\r\n Il n''y a pas convergence après %1.0f itérations.\r\n\r\n', ind);
           end
           infini = 1;
           break;
         end
         fx(n+1) = fx0;
         x(n+1) = x0;
         tol = abs(x0 - x(n));
         tolrel = tol/(abs(x0)+eps);
         if (tolrel < epsilon)
            break;
         end
    end
    if (tolrel < epsilon)
      err = abs(x - x0);
      converg = 1;
    else
      err = [];
      converg = 0;
      if (infini == 0)
        fprintf('\r\n Il n''y a pas de convergence après %1.0f iterations.\r\n\r\n', nmax);
      end
    end 

    if (nargin > 5)
      newton_out(g, dg, resul, x, fx, nmax, epsilon, converg);
    end
