    function  [t, y] = point_milieu(f, t0, y0, h, nmax, resul, mod)
%
%  Méthode du point milieu (Runge-Kutta d'ordre 2) pour
%  les systèmes d'équations différentielles de la forme y' = f(t,y)       
%  Programmeur: A. Lioret 
%  Référence: Analyse numérique pour ingénieurs, A. Fortin,
%             Presses internationales Polytechnique, 
%             Cinquième édition, 2015
%             Section 7.4.2
%
%  function  [t, y] = point_milieu(f, t0, y0, h, nmax, resul, mod)
%  Arguments
%
%  Entrée:
%  1) f: Le nom entre apostrophes (' ') du fichier .m contenant la fonction
%     f(t,y). Exemple: 'eqdif' (correspondant à un fichier eqdif.m) 
%     ou encore un système d'équations différentielles 
%     (voir systeme_eq_diff.m) par exemple
%     ou encore le nom de la fonction anonyme comme par exemple 
%     eq_diff_ano = @(t,y) -y(1) + t + 1.;
%     pour un système d'équations différentielles on aurait par exemple
%     eq_diff_ano = @(t,y) [-y(1) + y(2) + t + 1. ; y(1)*y(2)-t]
%  2) t0: Le temps de départ.
%  3) y0: La(les) condition(s) initiale(s). y0 doit
%     être un vecteur ligne, par exemple [1 2]. La dimension de y0
%     donne automatiquement la dimension du système.
%  4) h: Le pas de temps.
%  5) nmax: Le nombre maximal de pas de temps.
%  6) resul: (facultatif) Si vous voulez les résultats dans un fichier, 
%     vous devez fournir le nom entre apostrophes (' ') de ce fichier.
%     Exemple: 'resul.dat' qui créera un fichier nommé resul.dat.
%  7) mod: (facultatif) Si vous avez fourni un fichier de résultats, vous 
%     pouvez choisir le pas des iterations affichées dans ce fichier.
%     Exemple: si vous inscrivez 5, seulement les itérations qui
%              sont des multiples de 5 seront affichées dans le 
%              fichier de résultats.  
%     Si rien n'est écrit, mod = 1.
%  Sortie:
%  1) t: Vecteur contenant les temps ti.
%  2) y: Matrice contenant la solution obtenue.
%     La colonne i de y correspond à la solution de l'équation i.
%
%  Exemples d'appel:
%  [t,y] = point_milieu('eq_diff', 0.0, [1], 1.0e-1, 10)
%  [t,y] = point_milieu('eq_diff', 0.0, [1], 1.0e-1, 10, 'resul.dat')
%  [t,y] = point_milieu('eq_diff', 0.0, [1], 1.0e-1, 10, 'resul.dat', 5)
%
%  Si vous avec une fonction anonyme (@) (on ne met pas les apostrophes)
%  [t,y] = point_milieu(eq_diff_ano, 0.0, [1], 1.0e-1, 10)
%  [t,y] = point_milieu(eq_diff_ano, 0.0, [1], 1.0e-1, 10, 'resul.dat')
%  [t,y] = point_milieu(eq_diff_ano, 0.0, [1], 1.0e-1, 10, 'resul.dat', 5)

    g = f;
    f=fcnchk(f);
    nbeq = size(y0,2);
    y0 = y0';
    y = zeros(1,nbeq); t = zeros(1);
    n = 1;
    t(1) = t0;
    y(1,:) = y0';
    while(n <= nmax)
       k1 = h*f(t(n),y0);
       if (any(~isfinite(k1)))
         disp('La fonction n''est pas définie en certains points');
         return;
       end
       fyn = f(t(n)+h/2,y0+k1/2);
       if (any(~isfinite(fyn)))
         disp('La fonction n''est pas définie en certains points');
         return;
       end
       y0 = y0 + h*fyn;
       t(n+1) = t(n) + h;
       y(n+1,:) = y0';
       n = n+1;
    end
    
    if (nargin > 5)
      if (nargin < 7)
        mod = 1;
      end
      eq_diff_out(g, resul, t, y, h, nmax, 'du point milieu',mod);
    end
