    function  [x, err] = points_fixes(g, x0, nmax, epsilon, resul)
%
%  Méthode des points fixes en une variable pour résoudre x = g(x)
%  Programmeur: A. Fortin 
%  Référence: Analyse numérique pour ingénieurs, A. Fortin,
%             Presses internationales Polytechnique, 
%             Cinquième édition, 2015
%             Section 2.3
%
%
%  function  [x, err] = points_fixes(g, x0, nmax, epsilon, resul)
%  Arguments
%
%  Entrée:
%  1) g: Le nom entre apostrophes (' ') du fichier .m contenant la fonction 
%     g(x) comme par exemple  'g' (correspondant à un fichier g.m) 
%     ou encore le nom de la fonction anonyme comme par exemple 
%     g_anonyme = @(x) exp(-x);
%  2) x0: Un estimé initial de la racine.
%  3) nmax: Le nombre maximal d'itérations.
%  4) epsilon: Le critère d'arrêt.
%  5) resul: (facultatif) Si vous voulez les résultats dans un fichier, 
%     vous devez  fournir le nom entre apostrophes (' ') de ce fichier.
%     Exemple: 'resul.dat' qui créera un fichier nommé resul.dat.
%  Sortie:
%  1) x: Vecteur contenant les différentes itérations.
%  2) err: S'il y a convergence, err est le vecteur dont les composantes 
%     sont |x_n - r|, r étant le point fixe trouvé. 
%     Sinon, err est une matrice vide.
%
%  Exemples d'appel:
%  [x, err] = points_fixes('g', 2.0,  10, 1.0e-6)
%  [x, err] = points_fixes('g', 2.0,  10, 1.0e-6, 'resul.dat')
%
%  Si vous avec une fonction anonyme (@) (on ne met pas les apostrophes)
%  [x, err] = points_fixes(g_anonyme, 2.0,  10, 1.0e-6)
%  [x, err] = points_fixes(g_anonyme, 2.0,  10, 1.0e-6, 'resul.dat')

    f = g;
    g=fcnchk(g);
    infini = 0;
    x=zeros(1); gx=zeros(1); errel=zeros(1);
    x(1) = x0;
    gx0 = g(x0);
    if (~isfinite(gx0))
      error('g(x0) n''est pas un nombre fini.');
    end
    gx(1) = gx0;
    for n=1:nmax
         ind = n-1;
         x0 = gx0;
         gx0 = g(x0);
         if (~isfinite(gx0))
           if (n == 1)
             fprintf('\r\n Il n''y a pas convergence.\r\n\r\n');
           else
             fprintf('\r\n Il n''y a pas convergence après %1.0f itérations.\r\n\r\n', ind);
           end
           infini = 1;
           break;
         end
         x(n+1) = x0;
         gx(n+1) = gx0;
         tol = abs(x0 - x(n));
         tolrel = tol/(abs(x0)+eps);
         errel(n) = tolrel;        
         if (tolrel < epsilon)
            break;
         end
    end
    if (tolrel < epsilon)
      err = abs(x - x0);
      converg = 1;
    else
      err = [];
      converg = 0;
      if (infini == 0)
        fprintf('\r\n Il n''y a pas convergence après %1.0f itérations.\r\n\r\n', nmax);
      end
    end

    if (nargin > 4)
      points_fixes_out(f, resul, x, gx, errel, nmax, epsilon, converg);
    end
