    function  [x, err] = secante(f, x0, x1, nmax, epsilon, resul)
%
%  Méthode de la sécante en une variable pour résoudre f(x) = 0
%  Programmeur: A. Fortin 
%  Référence: Analyse numérique pour ingénieurs, A. Fortin,
%             Presses internationales Polytechnique, 
%             Cinquième édition, 2015
%             Section 2.5
%
%  function  [x, err] = secante(f, x0, x1, nmax, epsilon, resul)
%  Arguments
%
%  Entrée:
%  1) f: Le nom entre apostrophes (' ') du fichier .m contenant la fonction 
%     f(x) comme par exemple  'fonc' (correspondant à un fichier fonc.m) 
%     ou encore le nom de la fonction anonyme comme par exemple 
%     f_anonyme = @(x) x*x - 2.;
%  2) x0: Un premier estimé initial de la racine.
%  3) x1: Un deuxième estimé initial de la racine.
%  4) nmax: Le nombre maximal d'itérations.
%  5) epsilon: Le critère d'arrêt.
%  6) resul: (facultatif) Si vous voulez les résultats dans un fichier, 
%     vous devez fournir le nom entre apostrophes (' ') de ce fichier.
%     Exemple: 'resul.dat' qui créera un fichier nommé resul.dat.
%  Sortie:
%  1) x: Vecteur contenant les différentes itérations.
%  2) err: S'il y a convergence, err est le vecteur dont les composantes 
%     sont |x_n - r|,r étant la racine trouvée. 
%     Sinon, err est une matrice vide.
%
%  Exemples d'appels
%  Si vous avez un fichier fonction (par exemple fonc.m)
%  [x, err] = secante('fonc', 1, 2, 20, 1.0e-5)
%  [x, err] = secante('fonc', 1, 2, 20, 1.0e-5, 'resul.dat')
%   
%  Si vous avec une fonction anonyme (@) (on ne met pas les apostrophes)
%  [x, err] = secante( f_anonyme,  1, 2, 20, 1.0e-5)
%  [x, err] = secante( f_anonyme,  1, 2, 20, 1.0e-5, 'resul.dat')

    g=f;
    f=fcnchk(f);
    infini = 0;
    x(1) = x0;
    x(2) = x1;
    fx0 = f(x0);
    if (~isfinite(fx0))
      error('f(x0) n''est pas un nombre fini.');
    end
    fx1 = f(x1);
    if (~isfinite(fx1))
      error('f(x1) n''est pas un nombre fini.');
    end
    fx(1) = fx0;
    fx(2) = fx1;
    for n=2:nmax
         ind = n-1;
         dfx = (fx1-fx0) /(x1 - x0);
         x0 = x1;
         if (dfx ~= 0)            
            x1 = x1 - fx1/dfx;
         elseif (n == 1)
            error('(f(x1) - f(x0))/(x1 - x0) = 0');
         end
         fx0 = fx1;
         fx1 = f(x1);
         if (~isfinite(fx1))
           fprintf('\r\n Il n''y a pas convergence après %1.0f itérations.\r\n\r\n', ind);
           infini = 1;
           break;
         end
         x(n+1) = x1;
         fx(n+1) = fx1;
         tol = abs(x1 - x(n));
         tolrel = tol/(abs(x1)+eps);
         if (tolrel < epsilon)
            break;
         end
    end
    if (tolrel < epsilon)
      err = abs(x - x1);
      converg = 1;
    else
      err = [];
      converg = 0;
      if (infini == 0)
        fprintf('\r\nIl n''y a pas convergence après %1.0f itérations.\r\n\r\n', nmax);
      end
    end

    if (nargin > 5)
      secante_out(g, resul, x, fx, nmax, epsilon, converg);
    end
