    function  [x, R] = systeme_non_lineaire(f, x0, nmax, h, epsilon, resul)
%
%  Méthode de Newton pour les systèmes non linéaires R(x) = 0
%  Programmeur: A. Fortin 
%  Référence: Analyse numérique pour ingénieurs, A. Fortin,
%             Presses internationales Polytechnique, 
%             Cinquième édition, 2015
%             Section 3.10
%
%  function  [x, R] = systeme_non_lineaire(f, x0, nmax, h, epsilon, resul)
%
%  Entrée:
%  1) f: Le nom entre apostrophes (' ') du fichier .m contenant le résidu
%      R(x). Exemple: 'sys' (correspondant à un fichier sys.m) ou encore 
%     une fonction anonyme comme par exemple
%     sys_anonyme = @(x) [exp(x(1)) - x(2) ; x(1).^2 + x(2).^2 - 16 ]
%  2) x0: Un vecteur initial de départ de l'algorithme. x0 doit être un
%     vecteur ligne, par exemple [1 2]. La dimension de x0 donne
%     automatiquement la dimension du système non linéaire.
%  3) nmax: Le nombre maximal d'itérations.
%  4) h: Le paramètre h qui permet d'approximer
%     la matrice jacobienne par des différences finies.
%  5) epsilon: Le critère d'arrêt.
%  6) resul: (facultatif) Si vous voulez les résultats dans un fichier, 
%     vous devez fournir le nom entre apostrophes (' ') de ce fichier.
%     Exemple: 'resul.dat' qui créera un fichier nommé resul.dat.
%  Sortie:
%  1) x: Matrice contenant les différentes itérations 
%     (chaque ligne de x correspond à une itération).
%  2) R: est un vecteur contenant la norme du résidu à chaque itération
%
%  Préalable:
%  Vous devez créer un fichier .m contenant le vecteur residu R(x).
%  Voir par exemple le fichier sys.m. Alternativement, vous pouvez aussi
%  utiliser une fonction anonyme.
%
%  Exemples d'appel:
%  [x, R] = systeme_non_lineaire('sys', [1 2], 10, 1.0e-3, 1.0e-6)
%  [x, R] = systeme_non_lineaire('sys', [1 2], 10, 1.0e-3, 1.0e-6, 'resul.dat')
%
%  [x, R] = systeme_non_lineaire(sys_anonyme, [1 2], 10, 1.0e-3, 1.0e-6)
%  [x, R] = systeme_non_lineaire(sys_anonyme, [1 2], 10, 1.0e-3, 1.0e-6, 'resul.dat')

    g=f;
    f=fcnchk(f);
    nbeq = size(x0,2);
    x0 = x0';
    jac = zeros(nbeq);
    x = zeros(1,nbeq);
    infini = 0;
    b = (-f(x0));
    x(1,:) = x0';
    R = norm(b);
    if (~isfinite(R))
      error('La norme de R(x0) n''est pas un nombre fini.');
    end
    for n=1:nmax
       ind = n-1;
       for k =1: nbeq
          acc = zeros(nbeq,1);
          acc(k) = h;
          % acc(k) = sqrt(eps)*abs(x0(k));
          % on passe par les nombres complexes
           jac(:,k) = imag(f(x0 + 1i*acc))./acc(k);
          % On passe par les différences finies
          % jac(:,k) =  (f(x0 + acc) - f(x0 - acc))./(2*acc(k));
       end
       detjac = det(jac);
       if (~any(any(isfinite(jac))) || ~isfinite(detjac))
         disp(' Matrice jacobienne =');
         disp(jac);
         disp(' La matrice jacobienne contient des éléments qui sont NaN');
         disp('  (NaN = Not-a-Number)');
         return
       elseif (detjac ~= 0)
         delta = jac \ b;
         x0 = x0 + delta;
       else
         disp(' La matrice jacobienne n''est pas inversible');
         return
       end
       b = (-f(x0));
       test = norm(b);
       if (~isfinite(test))
         if (n == 1)
           fprintf('\r\n Il n''y a pas convergence.\r\n\r\n');
         else
           fprintf('\r\n Il n''y a pas convergence après %1.0f itérations.\r\n\r\n', ind);
         end
         infini = 1;
         break;
       end
       R = [R; test];
       tol = norm(delta);
       tolrel = tol/(norm(x0)+eps);
       x(n+1,:) = x0';
       if (tolrel < epsilon && test <= epsilon)
         break;
       end
    end
    if (tolrel < epsilon && test <= epsilon)
      converg = 1;
    else
      converg = 0;
      if (infini == 0)
        fprintf('\r\n Il n''y a pas de convergence après %1.0f iterations.\r\n\r\n', nmax);
      end
    end
    
    if (nargin > 5)
      systeme_non_lineaire_out(g, resul, x, R, nmax, h, epsilon, converg);
    end
