    function [t,y] = tir(a2, a1, a0, ab, yayb, n, methode, type, resul, mod)
%
%  Méthode de tir
%  Programmeur: A. Fortin 
%  Référence: Analyse numérique pour ingénieurs, A. Fortin,
%             Presses internationales Polytechnique, 
%             Cinquième édition, 2015
%             Section 7.9.1
%
%  function [t,y] = tir(a2, a1, a0, ab, yayb, n, methode, type, resul, mod)
%  Arguments
%
%  Entree:
%  1) a2: Le nom entre apostrophes (' ') du fichier .m contenant
%     la fonction a2(x).
%     Exemple: 'a2' (correspondant à un fichier a2.m).
%     ou encore une fonction anonyme
%     a2_ano = @(x) -1./x ;
%  2) a1: Le nom entre apostrophes (' ') du fichier .m contenant
%     la fonction a1(x).
%     Exemple: 'a1' (correspondant à un fichier a1.m).
%     ou encore une fonction anonyme
%     a1_ano = @(x) 0 ;
%  3) a0: Le nom entre apostrophes (' ') du fichier .m contenant
%     la fonction a0(x).
%     Exemple: 'a0' (correspondant à un fichier a0.m).
%     ou encore une fonction anonyme
%     a0_ano = @(x) 0 ;
%  4) ab: L'intervalle de résolution dans un
%     vecteur ligne. Exemple: [1 2].
%  5) yayb: Les conditions aux limites dans un vecteur ligne.
%     Exemple: [0 0.693147] tel que y(a) = 0 et y(b) = 0.693147.
%  6) n: Le nombre de pas de temps.
%  7) methode: Vous devez fournir la méthode à utiliser pour la
%     résolution numérique des systèmes d'équations différentielles
%     intermédiaires, soit:
%             'euler' pour la méthode d'Euler explicite
%             'eulmo' pour la méthode d'Euler modifiée 
%             'ptmil' pour la méthode du point milieu
%             'runk4' pour la méthode de Runge-Kutta d'ordre 4
%  8) type:Si type = 1, condition aux limites en x=b de la forme y(b)= y_b
%          Si type = 2, condition aux limites en x=b de la forme y'(b)=y'_b
%  9) resul: (facultatif) Si vous voulez les résultats dans un fichier, 
%     vous devez fournir le nom entre apostrophes (' ') de ce fichier.
%     Exemple: 'resul.dat' qui créera un fichier nomme resul.dat.
%  10) mod: (facultatif)  Si vous avez fourni un fichier de résultats, vous 
%     pouvez choisir le pas des itérations affichées dans ce fichier.
%     Exemple: si vous inscrivez 5, seulement les itérations qui
%              sont des multiples de 5 seront affichées dans le 
%              fichier de résultats.  
%     Si rien n'est écrit, mod = 1.
%  Sortie:
%  1) t: Vecteur contenant les temps ti.
%  2) Y: Matrice contenant respectivement en colonne,
%     la solution y, y_1, y'_1, y_2 et y'_2 pour chaque temps ti.
%
%  Exemples d'appel:
%  [t,Y] = tir('a2','a1','a0', [1 2], [100 20], 10, 'runk4',1)
%  [t,Y] = tir('a2','a1','a0', [1 2], [100 20], 10, 'runk4',1, 'resul.dat')
%  [t,Y] = tir('a2','a1','a0', [1 2], [100 20], 10, 'runk4',1, 'resul.dat', 5)
%
%  Si vous avez créé des fonctions anonymes
%  [t,Y] = tir(a2_ano, a1_ano, a0_ano, [1 2], [100 20], 10,'runk4',1)
%  [t,Y] = tir(a2_ano, a1_ano, a0_ano, [1 2], [100 20], 10,'runk4',1, 'resul.dat')
%  [t,Y] = tir(a2_ano, a1_ano, a0_ano, [1 2], [100 20], 10,'runk4',1, 'resul.dat', 5)

    b2=a2; b1=a1; b0=a0;
    a2=fcnchk(a2); a1=fcnchk(a1); a0=fcnchk(a0);
    U = @(x,u) [u(2); a2(x)*u(2)+a1(x)*u(1)+a0(x)];
    

    h = (ab(2) - ab(1))/n;
    if (methode == 'euler')
      [t, yu] = euler_explicite(U, ab(1), [yayb(1) 0], h, n);
      [t, yv] = euler_explicite(U, ab(1), [yayb(1) 1], h, n);
    elseif (methode == 'eulmo')
      [t, yu] = euler_modifiee(U, ab(1), [yayb(1) 0], h, n);
      [t, yv] = euler_modifiee(U, ab(1), [yayb(1) 1], h, n);
    elseif (methode == 'ptmil')
      [t, yu] = point_milieu(U, ab(1), [yayb(1) 0], h, n);
      [t, yv] = point_milieu(U, ab(1), [yayb(1) 1], h, n);
    elseif (methode == 'runk4')
      [t, yu] = runge_kutta4(U, ab(1), [yayb(1) 0], h, n);
      [t, yv] = runge_kutta4(U, ab(1), [yayb(1) 1], h, n);
    end

    y1 = yu(:,1);
    y1p = yu(:,2);
    y2 = yv(:,1);
    y2p = yv(:,2);
    y1b = y1(n+1);
    y2b = y2(n+1);
    y1bp = y1p(n+1);
    y2bp = y2p(n+1);
    if(type == 1)
       coef(1) = (yayb(2) - y2b)/(y1b-y2b);
       coef(2) = (y1b - yayb(2))/(y1b-y2b);
       ys = coef(1)*y1 + coef(2)*y2;
    end
    if(type == 2)
       coef(1) = (yayb(2) - y2bp)/(y1bp-y2bp);
       coef(2) = (y1bp - yayb(2))/(y1bp-y2bp);
       ys = coef(1)*y1 + coef(2)*y2;
    end
    
    y = [ys y1 y1p y2 y2p];


    if (nargin > 8)
      if (nargin < 10)
        mod = 1;
      end
      tir_out(resul, b2, b1, b0, ab, yayb, t, y, coef, n, methode, type, mod);
    end
