   function [t, y, beta] = tir_non_lineaire(systir, type_cond, ab, yayb,...
             n, methode, int_beta, resul, mod)
%
%----------------------------------------|
% Analyse numérique pour ingénieurs      |
% André Fortin                           |
% Presses internationales Polytechnique  |
% Cinquième édition, 2015                |
%-----------------------------------------
% Méthode de tir (cas non linéaire) : section 7.9.2
% -----------------------------
%
%  function [t, y, beta] = tir_non_lineaire(systir, type_cond, ab, yayb, n, methode, int_beta, resul, mod)
%  Arguments
%
%  Entrée:
%  1) systir: Le nom entre apostrophes (' ') du fichier .m contenant
%     le système de deux équations d'ordre 1 à résoudre.
%     Exemple: 'systeme_tir' (correspondant à un fichier systeme_tir.m)
%     ou encore une fonction anonyme comme par exemple
%     systeme_tir_ano = @(x,y) [y(2) ; -(2.0 ./ x) * y(2) + 1 ./(x.^2)]
%  2) type-cond:  donne le type de conditions aux limites.
%        type_cond = 1: conditions aux limites y(a) et y(b)
%        type_cond = 2: conditions aux limites y(a) et y'(b)
%  3) ab: L'intervalle de définition dans un vecteur ligne.
%     Exemple: [1 2].
%  4) yayb: Les conditions aux limites dans un vecteur ligne.
%     Exemple: [0 0.693147] tel que y(a) = 0 et y(b) = 0.693147.
%  5) n: Le nombre de pas de temps.
%  6) methode: Vous devez fournir la méthode à utiliser pour la
%     résolution numérique des systèmes d'équations différentielles
%     intermédiaires, soit:
%             'euler' pour la méthode d'Euler
%             'eulmo' pour la méthode d'Euler modifiee 
%             'ptmil' pour la méthode du point milieu
%             'runk4' pour la méthode de Runge-Kutta d'ordre 4
%  7) int_beta: Intervalle de départ pour la méthode de la sécante en beta
%  8) resul: (facultatif) Si vous voulez les résultats dans un fichier, 
%     vous devez fournir le nom entre apostrophes (' ') de ce fichier.
%     Exemple: 'resul.dat' qui créera un fichier nomme resul.dat.
%  9) mod: (facultatif) Si vous avez fourni un fichier de résultats, vous 
%     pouvez choisir le pas des itérations affichées dans ce fichier.
%     Exemple: si vous inscrivez 5, seulement les itérations qui
%              sont des multiples de 5 seront affichées dans le 
%              fichier de résultats.  
%     Si rien n'est écrit, mod = 1.
%  Sortie:
%  1) t: Vecteur contenant les temps ti.
%  2) Y: Matrice contenant respectivement en colonne,
%     la solution y, y_1, y'_1  pour chaque temps ti.
%  3) beta: Les différentes valeurs de beta (angles de tir)
%
%  Exemples d'appel:
%  [t,y, beta] = tir_non_lineaire('systeme_tir', 1, [1 2], [0 0.693147], 10, 'runk4', [0 5])
%  [t,y, beta] = tir_non_lineaire('systeme_tir', 1, [1 2], [0 0.693147], 10, 'runk4', [0 5], 'resul.dat')
%  [t,y, beta] = tir_non_lineaire('systeme_tir', 1, [1 2], [0 0.693147], 10, 'runk4', [0 5], 'resul.dat', 5)
%
%  [t,y, beta] = tir_non_lineaire(systeme_tir_ano, 1, [1 2], [0 0.693147], 10, 'runk4', [0 5])
%  [t,y, beta] = tir_non_lineaire(systeme_tir_ano, 1, [1 2], [0 0.693147], 10, 'runk4', [0 5], 'resul.dat')
%  [t,y, beta] = tir_non_lineaire(systeme_tir_ano, 1, [1 2], [0 0.693147], 10, 'runk4', [0 5], 'resul.dat', 5)
    
    h = (ab(2) - ab(1))/n;
    beta0 = int_beta(1);
    beta1 = int_beta(2);
    if(type_cond == 1 )
       condini11 = yayb(1);
       condini12 = beta0;
       condini21 = yayb(1);
       condini22 = beta1;
    elseif(type_cond==2)
       condini11 = yayb(1);
       condini12 = beta0;
       condini21 = yayb(1);
       condini22 = beta1;
    elseif(type_cond==3)
       condini11 = beta0;
       condini12 = yayb(1);
       condini21 = beta1;
       condini22 = yayb(1);
    elseif(type_cond==4)
       condini11 = beta0;
       condini12 = yayb(1);
       condini21 = beta1;
       condini22 = yayb(1);
    end
    beta(1) = beta0;
    beta(2) = beta1;
    if (methode == 'euler')
      [t, y0] = euler_explicite(systir, ab(1), [condini11 condini12], h, n);
      [t, y1] = euler_explicite(systir, ab(1), [condini21 condini22], h, n);
    elseif (methode == 'eulmo')
      [t, y0] = euler_modifiee(systir, ab(1), [condini11 condini12], h, n);
      [t, y1] = euler_modifiee(systir, ab(1), [condini21 condini22], h, n);
    elseif (methode == 'ptmil')
      [t, y0] = point_milieu(systir, ab(1), [condini11 condini12], h, n);
      [t, y1] = point_milieu(systir, ab(1), [condini21 condini22], h, n);
    elseif (methode == 'runk4')
      [t, y0] = runge_kutta4(systir, ab(1), [condini11 condini12], h, n);
      [t, y1] = runge_kutta4(systir, ab(1), [condini21 condini22], h, n);
    end
    
    if(type_cond==1)
       f0 = y0(n+1,1) - yayb(2);
       f1 = y1(n+1,1) - yayb(2);
    elseif (type_cond==2)
       f0 = y0(n+1,2) - yayb(2);
       f1 = y1(n+1,2) - yayb(2);
    elseif (type_cond==3)
       f0 = y0(n+1,1) - yayb(2);
       f1 = y1(n+1,1) - yayb(2);
    elseif (type_cond==4)
       f0 = y0(n+1,2) - yayb(2);
       f1 = y1(n+1,2) - yayb(2);
    end
    converge = 0;
    crit_conv = 1.0e-10;
    for iter=2:30
       dfx = f1*(beta1 - beta0)/(f1 - f0);
       beta0 = beta1;
       if(abs(dfx) > eps)
          beta1 = beta1 - dfx;
       end
       f0 = f1;
       beta(iter) = beta1;
       if(type_cond == 1 )
          condini21 = yayb(1);
          condini22 = beta1;
       elseif(type_cond==2)
          condini21 = yayb(1);
          condini22 = beta1;
       elseif(type_cond==3)
          condini21 = beta1;
          condini22 = yayb(1);
       elseif(type_cond==4)
          condini21 = beta1;
          condini22 = yayb(1);
       end
    
       if (methode == 'euler')
         [t, y] = euler_explicite(systir, ab(1), [condini21 condini22], h, n);
       elseif (methode == 'eulmo')
         [t, y] = euler_modifiee(  systir, ab(1), [condini21 condini22], h, n);
       elseif (methode == 'ptmil')
         [t, y] = point_milieu(systir, ab(1), [condini21 condini22], h, n);
       elseif (methode == 'runk4')
         [t, y] = runge_kutta4(     systir, ab(1), [condini21 condini22], h, n);
       end
       if(type_cond==1)
          f2 = y(n+1,1) - yayb(2);
       elseif (type_cond==2)
          f2 = y(n+1,2) - yayb(2);
       elseif (type_cond==3)
          f2 = y(n+1,1) - yayb(2);
       elseif (type_cond==4)
          f2 = y(n+1,2) - yayb(2);
       end
       f1 = f2;
       if(abs(f2) < crit_conv)
         %  beta1;
         %  f1;
           converge = 1;
           break;
       end
    end
    if(converge ==0)
        disp('La méthode de la sécante n''a pas convergé en 30 itérations')
    end
    
    if (nargin > 7)
      if (nargin < 9)
        mod = 1;
      end
      eq_diff_out(systir, resul, t, y,  h, n, 'tir non linéaire', mod)
    end
